@@ -46,6 +46,7 @@
 #define MGMT_STATUS_CANCELLED		0x10
 #define MGMT_STATUS_INVALID_INDEX	0x11
 #define MGMT_STATUS_RFKILLED		0x12
+#define MGMT_STATUS_ALREADY_PAIRED	0x13
 
 struct mgmt_hdr {
 	uint16_t opcode;
@@ -98,6 +99,7 @@ struct mgmt_rp_read_index_list {
 #define MGMT_SETTING_DEBUG_KEYS		0x00001000
 #define MGMT_SETTING_PRIVACY		0x00002000
 #define MGMT_SETTING_CONFIGURATION	0x00004000
+#define MGMT_SETTING_STATIC_ADDRESS	0x00008000
 
 #define MGMT_OP_READ_INFO		0x0004
 struct mgmt_rp_read_info {
@@ -602,7 +604,7 @@ struct mgmt_ev_new_irk {
 
 struct mgmt_csrk_info {
 	struct mgmt_addr_info addr;
-	uint8_t master;
+	uint8_t type;
 	uint8_t val[16];
 } __packed;
 
@@ -756,6 +758,7 @@ static const char *mgmt_status[] = {
 	"Cancelled",
 	"Invalid Index",
 	"Blocked through rfkill",
+	"Already Paired",
 };
 
 #ifndef NELEM
