@@ -30,9 +30,9 @@
 #include <stdlib.h>
 #include <errno.h>
 
+#include "lib/bluetooth.h"
 #include "uuid.h"
 
-#if __BYTE_ORDER == __BIG_ENDIAN
 static uint128_t bluetooth_base_uuid = {
 	.data = {	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00,
 			0x80, 0x00, 0x00, 0x80, 0x5F, 0x9B, 0x34, 0xFB }
@@ -41,33 +41,36 @@ static uint128_t bluetooth_base_uuid = {
 #define BASE_UUID16_OFFSET	2
 #define BASE_UUID32_OFFSET	0
 
-#else
-static uint128_t bluetooth_base_uuid = {
-	.data = {	0xFB, 0x34, 0x9B, 0x5F, 0x80, 0x00, 0x00, 0x80,
-			0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }
-};
-
-#define BASE_UUID16_OFFSET	12
-#define BASE_UUID32_OFFSET	BASE_UUID16_OFFSET
-
-#endif
-
 static void bt_uuid16_to_uuid128(const bt_uuid_t *src, bt_uuid_t *dst)
 {
+	uint16_t be16;
+
 	dst->value.u128 = bluetooth_base_uuid;
 	dst->type = BT_UUID128;
 
-	memcpy(&dst->value.u128.data[BASE_UUID16_OFFSET],
-			&src->value.u16, sizeof(src->value.u16));
+	/*
+	 * No matter the system: 128-bit UUIDs should be stored
+	 * as big-endian. 16-bit UUIDs are stored on host order.
+	 */
+
+	be16 = htons(src->value.u16);
+	memcpy(&dst->value.u128.data[BASE_UUID16_OFFSET], &be16, sizeof(be16));
 }
 
 static void bt_uuid32_to_uuid128(const bt_uuid_t *src, bt_uuid_t *dst)
 {
+	uint32_t be32;
+
 	dst->value.u128 = bluetooth_base_uuid;
 	dst->type = BT_UUID128;
 
-	memcpy(&dst->value.u128.data[BASE_UUID32_OFFSET],
-				&src->value.u32, sizeof(src->value.u32));
+	/*
+	 * No matter the system: 128-bit UUIDs should be stored
+	 * as big-endian. 32-bit UUIDs are stored on host order.
+	 */
+
+	be32 = htonl(src->value.u32);
+	memcpy(&dst->value.u128.data[BASE_UUID32_OFFSET], &be32, sizeof(be32));
 }
 
 void bt_uuid_to_uuid128(const bt_uuid_t *src, bt_uuid_t *dst)
@@ -82,6 +85,7 @@ void bt_uuid_to_uuid128(const bt_uuid_t *src, bt_uuid_t *dst)
 	case BT_UUID16:
 		bt_uuid16_to_uuid128(src, dst);
 		break;
+	case BT_UUID_UNSPEC:
 	default:
 		break;
 	}
@@ -154,10 +158,7 @@ int bt_uuid_to_string(const bt_uuid_t *uuid, char *str, size_t n)
 		unsigned int   data4;
 		unsigned short data5;
 
-		uint128_t nvalue;
-		const uint8_t *data = (uint8_t *) &nvalue;
-
-		hton128(&uuid->value.u128, &nvalue);
+		const uint8_t *data = (uint8_t *) &uuid->value.u128;
 
 		memcpy(&data0, &data[0], 4);
 		memcpy(&data1, &data[4], 2);
@@ -172,6 +173,7 @@ int bt_uuid_to_string(const bt_uuid_t *uuid, char *str, size_t n)
 				ntohl(data4), ntohs(data5));
 		}
 		break;
+	case BT_UUID_UNSPEC:
 	default:
 		snprintf(str, n, "Type of UUID (%x) unknown.", uuid->type);
 		return -EINVAL;	/* Enum type of UUID not set */
@@ -189,6 +191,19 @@ static inline int is_uuid128(const char *string)
 			string[23] == '-');
 }
 
+static inline int is_base_uuid128(const char *string)
+{
+	uint16_t uuid;
+	char dummy;
+
+	if (!is_uuid128(string))
+		return 0;
+
+	return sscanf(string,
+		"0000%04hx-0000-1000-8000-00805%1[fF]9%1[bB]34%1[fF]%1[bB]",
+		&uuid, &dummy, &dummy, &dummy, &dummy) == 5;
+}
+
 static inline int is_uuid32(const char *string)
 {
 	return (strlen(string) == 8 || strlen(string) == 10);
@@ -205,7 +220,7 @@ static int bt_string_to_uuid16(bt_uuid_t *uuid, const char *string)
 	char *endptr = NULL;
 
 	u16 = strtol(string, &endptr, 16);
-	if (endptr && *endptr == '\0') {
+	if (endptr && (*endptr == '\0' || *endptr == '-')) {
 		bt_uuid16_create(uuid, u16);
 		return 0;
 	}
@@ -231,8 +246,8 @@ static int bt_string_to_uuid128(bt_uuid_t *uuid, const char *string)
 {
 	uint32_t data0, data4;
 	uint16_t data1, data2, data3, data5;
-	uint128_t n128, u128;
-	uint8_t *val = (uint8_t *) &n128;
+	uint128_t u128;
+	uint8_t *val = (uint8_t *) &u128;
 
 	if (sscanf(string, "%08x-%04hx-%04hx-%04hx-%08x%04hx",
 				&data0, &data1, &data2,
@@ -253,8 +268,6 @@ static int bt_string_to_uuid128(bt_uuid_t *uuid, const char *string)
 	memcpy(&val[10], &data4, 4);
 	memcpy(&val[14], &data5, 2);
 
-	ntoh128(&n128, &u128);
-
 	bt_uuid128_create(uuid, u128);
 
 	return 0;
@@ -262,7 +275,9 @@ static int bt_string_to_uuid128(bt_uuid_t *uuid, const char *string)
 
 int bt_string_to_uuid(bt_uuid_t *uuid, const char *string)
 {
-	if (is_uuid128(string))
+	if (is_base_uuid128(string))
+		return bt_string_to_uuid16(uuid, string + 4);
+	else if (is_uuid128(string))
 		return bt_string_to_uuid128(uuid, string);
 	else if (is_uuid32(string))
 		return bt_string_to_uuid32(uuid, string);
@@ -271,3 +286,29 @@ int bt_string_to_uuid(bt_uuid_t *uuid, const char *string)
 
 	return -EINVAL;
 }
+
+int bt_uuid_strcmp(const void *a, const void *b)
+{
+	return strcasecmp(a, b);
+}
+
+int bt_uuid_to_le(const bt_uuid_t *src, void *dst)
+{
+	bt_uuid_t uuid;
+
+	switch (src->type) {
+	case BT_UUID16:
+		bt_put_le16(src->value.u16, dst);
+		return 0;
+	case BT_UUID32:
+		bt_uuid_to_uuid128(src, &uuid);
+		/* Fallthrough */
+	case BT_UUID128:
+		/* Convert from 128-bit BE to LE */
+		bswap_128(&src->value.u128, dst);
+		return 0;
+	case BT_UUID_UNSPEC:
+	default:
+		return -EINVAL;
+	}
+}
