@@ -28,6 +28,7 @@
 
 #include "src/shared/queue.h"
 #include "src/shared/att.h"
+#include "lib/bluetooth.h"
 #include "lib/uuid.h"
 #include "src/shared/gatt-helpers.h"
 #include "src/shared/util.h"
@@ -208,15 +209,16 @@ static bool convert_uuid_le(const uint8_t *src, size_t len, uint8_t dst[16])
 	return true;
 }
 
-struct discovery_op {
+struct bt_gatt_request {
 	struct bt_att *att;
+	unsigned int id;
 	uint16_t end_handle;
 	int ref_count;
 	bt_uuid_t uuid;
 	uint16_t service_type;
 	struct bt_gatt_result *result_head;
 	struct bt_gatt_result *result_tail;
-	bt_gatt_discovery_callback_t callback;
+	bt_gatt_request_callback_t callback;
 	void *user_data;
 	bt_gatt_destroy_func_t destroy;
 };
@@ -224,7 +226,7 @@ struct discovery_op {
 static struct bt_gatt_result *result_append(uint8_t opcode, const void *pdu,
 						uint16_t pdu_len,
 						uint16_t data_len,
-						struct discovery_op *op)
+						struct bt_gatt_request *op)
 {
 	struct bt_gatt_result *result;
 
@@ -247,7 +249,7 @@ bool bt_gatt_iter_next_included_service(struct bt_gatt_iter *iter,
 				uint16_t *end_handle, uint8_t uuid[16])
 {
 	struct bt_gatt_result *read_result;
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	const void *pdu_ptr;
 	int i = 0;
 
@@ -326,7 +328,7 @@ bool bt_gatt_iter_next_service(struct bt_gatt_iter *iter,
 				uint16_t *start_handle, uint16_t *end_handle,
 				uint8_t uuid[16])
 {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	const void *pdu_ptr;
 	bt_uuid_t tmp;
 
@@ -368,7 +370,7 @@ bool bt_gatt_iter_next_characteristic(struct bt_gatt_iter *iter,
 				uint16_t *value_handle, uint8_t *properties,
 				uint8_t uuid[16])
 {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	const void *pdu_ptr;
 
 	if (!iter || !iter->result || !start_handle || !end_handle ||
@@ -444,7 +446,7 @@ bool bt_gatt_iter_next_read_by_type(struct bt_gatt_iter *iter,
 				uint16_t *handle, uint16_t *length,
 				const uint8_t **value)
 {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	const void *pdu_ptr;
 
 	if (!iter || !iter->result || !handle || !length || !value)
@@ -533,13 +535,14 @@ done:
 		op->callback(success, att_ecode, op->user_data);
 }
 
-bool bt_gatt_exchange_mtu(struct bt_att *att, uint16_t client_rx_mtu,
+unsigned int bt_gatt_exchange_mtu(struct bt_att *att, uint16_t client_rx_mtu,
 					bt_gatt_result_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
 	struct mtu_op *op;
 	uint8_t pdu[2];
+	unsigned int id;
 
 	if (!att || !client_rx_mtu)
 		return false;
@@ -556,35 +559,12 @@ bool bt_gatt_exchange_mtu(struct bt_att *att, uint16_t client_rx_mtu,
 
 	put_le16(client_rx_mtu, pdu);
 
-	if (!bt_att_send(att, BT_ATT_OP_MTU_REQ, pdu, sizeof(pdu),
-							mtu_cb, op,
-							destroy_mtu_op)) {
+	id = bt_att_send(att, BT_ATT_OP_MTU_REQ, pdu, sizeof(pdu), mtu_cb, op,
+								destroy_mtu_op);
+	if (!id)
 		free(op);
-		return false;
-	}
 
-	return true;
-}
-
-static void put_uuid_le(const bt_uuid_t *src, void *dst)
-{
-	bt_uuid_t uuid;
-
-	switch (src->type) {
-	case BT_UUID16:
-		put_le16(src->value.u16, dst);
-		break;
-	case BT_UUID128:
-		bswap_128(&src->value.u128, dst);
-		break;
-	case BT_UUID32:
-		bt_uuid_to_uuid128(src, &uuid);
-		bswap_128(&uuid.value.u128, dst);
-		break;
-	case BT_UUID_UNSPEC:
-	default:
-		break;
-	}
+	return id;
 }
 
 static inline int get_uuid_len(const bt_uuid_t *uuid)
@@ -595,35 +575,73 @@ static inline int get_uuid_len(const bt_uuid_t *uuid)
 	return (uuid->type == BT_UUID16) ? 2 : 16;
 }
 
-static struct discovery_op* discovery_op_ref(struct discovery_op *op)
+struct bt_gatt_request *bt_gatt_request_ref(struct bt_gatt_request *req)
 {
-	__sync_fetch_and_add(&op->ref_count, 1);
+	if (!req)
+		return NULL;
 
-	return op;
+	__sync_fetch_and_add(&req->ref_count, 1);
+
+	return req;
 }
 
-static void discovery_op_unref(void *data)
+void bt_gatt_request_unref(struct bt_gatt_request *req)
 {
-	struct discovery_op *op = data;
-
-	if (__sync_sub_and_fetch(&op->ref_count, 1))
+	if (!req)
 		return;
 
-	if (op->destroy)
-		op->destroy(op->user_data);
+	if (__sync_sub_and_fetch(&req->ref_count, 1))
+		return;
 
-	result_destroy(op->result_head);
+	bt_gatt_request_cancel(req);
+
+	if (req->destroy)
+		req->destroy(req->user_data);
+
+	result_destroy(req->result_head);
+
+	free(req);
+}
+
+void bt_gatt_request_cancel(struct bt_gatt_request *req)
+{
+	if (!req)
+		return;
+
+	if (!req->id)
+		return;
+
+	bt_att_cancel(req->att, req->id);
+	req->id = 0;
+}
+
+static void async_req_unref(void *data)
+{
+	struct bt_gatt_request *req = data;
+
+	bt_gatt_request_unref(req);
+}
+
+static void discovery_op_complete(struct bt_gatt_request *op, bool success,
+								uint8_t ecode)
+{
+	if (op->callback)
+		op->callback(success, ecode, success ? op->result_head : NULL,
+								op->user_data);
+
+	if (!op->id)
+		async_req_unref(op);
+	else
+		op->id = 0;
 
-	free(op);
 }
 
 static void read_by_grp_type_cb(uint8_t opcode, const void *pdu,
 					uint16_t length, void *user_data)
 {
-	struct discovery_op *op = user_data;
+	struct bt_gatt_request *op = user_data;
 	bool success;
 	uint8_t att_ecode = 0;
-	struct bt_gatt_result *final_result = NULL;
 	struct bt_gatt_result *cur_result;
 	size_t data_length;
 	size_t list_length;
@@ -655,8 +673,8 @@ static void read_by_grp_type_cb(uint8_t opcode, const void *pdu,
 	data_length = ((uint8_t *) pdu)[0];
 	list_length = length - 1;
 
-	if ((list_length % data_length) ||
-				(data_length != 6 && data_length != 20)) {
+	if ((data_length != 6 && data_length != 20) ||
+					(list_length % data_length)) {
 		success = false;
 		goto done;
 	}
@@ -679,14 +697,14 @@ static void read_by_grp_type_cb(uint8_t opcode, const void *pdu,
 		put_le16(op->end_handle, pdu + 2);
 		put_le16(op->service_type, pdu + 4);
 
-		if (bt_att_send(op->att, BT_ATT_OP_READ_BY_GRP_TYPE_REQ,
-							pdu, sizeof(pdu),
-							read_by_grp_type_cb,
-							discovery_op_ref(op),
-							discovery_op_unref))
+		op->id = bt_att_send(op->att, BT_ATT_OP_READ_BY_GRP_TYPE_REQ,
+						pdu, sizeof(pdu),
+						read_by_grp_type_cb,
+						bt_gatt_request_ref(op),
+						async_req_unref);
+		if (op->id)
 			return;
 
-		discovery_op_unref(op);
 		success = false;
 		goto done;
 	}
@@ -701,22 +719,18 @@ static void read_by_grp_type_cb(uint8_t opcode, const void *pdu,
 				cur_result->pdu + length - data_length + 1);
 
 success:
-	/* End of procedure */
-	final_result = op->result_head;
 	success = true;
 
 done:
-	if (op->callback)
-		op->callback(success, att_ecode, final_result, op->user_data);
+	discovery_op_complete(op, success, att_ecode);
 }
 
 static void find_by_type_val_cb(uint8_t opcode, const void *pdu,
 					uint16_t length, void *user_data)
 {
-	struct discovery_op *op = user_data;
+	struct bt_gatt_request *op = user_data;
 	bool success;
 	uint8_t att_ecode = 0;
-	struct bt_gatt_result *final_result = NULL;
 	uint16_t last_end;
 
 	if (opcode == BT_ATT_OP_ERROR_RSP) {
@@ -757,46 +771,43 @@ static void find_by_type_val_cb(uint8_t opcode, const void *pdu,
 		put_le16(last_end + 1, pdu);
 		put_le16(op->end_handle, pdu + 2);
 		put_le16(op->service_type, pdu + 4);
-		put_uuid_le(&op->uuid, pdu + 6);
+		bt_uuid_to_le(&op->uuid, pdu + 6);
 
-		if (bt_att_send(op->att, BT_ATT_OP_FIND_BY_TYPE_VAL_REQ,
-							pdu, sizeof(pdu),
-							find_by_type_val_cb,
-							discovery_op_ref(op),
-							discovery_op_unref))
+		op->id = bt_att_send(op->att, BT_ATT_OP_FIND_BY_TYPE_VAL_REQ,
+						pdu, sizeof(pdu),
+						find_by_type_val_cb,
+						bt_gatt_request_ref(op),
+						async_req_unref);
+		if (op->id)
 			return;
 
-		discovery_op_unref(op);
 		success = false;
 		goto done;
 	}
 
 success:
-	/* End of procedure */
-	final_result = op->result_head;
 	success = true;
 
 done:
-	if (op->callback)
-		op->callback(success, att_ecode, final_result, op->user_data);
+	discovery_op_complete(op, success, att_ecode);
 }
 
-static bool discover_services(struct bt_att *att, bt_uuid_t *uuid,
+static struct bt_gatt_request *discover_services(struct bt_att *att,
+					bt_uuid_t *uuid,
 					uint16_t start, uint16_t end,
-					bt_gatt_discovery_callback_t callback,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy,
 					bool primary)
 {
-	struct discovery_op *op;
-	bool result;
+	struct bt_gatt_request *op;
 
 	if (!att)
-		return false;
+		return NULL;
 
-	op = new0(struct discovery_op, 1);
+	op = new0(struct bt_gatt_request, 1);
 	if (!op)
-		return false;
+		return NULL;
 
 	op->att = att;
 	op->end_handle = end;
@@ -814,17 +825,17 @@ static bool discover_services(struct bt_att *att, bt_uuid_t *uuid,
 		put_le16(end, pdu + 2);
 		put_le16(op->service_type, pdu + 4);
 
-		result = bt_att_send(att, BT_ATT_OP_READ_BY_GRP_TYPE_REQ,
-							pdu, sizeof(pdu),
-							read_by_grp_type_cb,
-							discovery_op_ref(op),
-							discovery_op_unref);
+		op->id = bt_att_send(att, BT_ATT_OP_READ_BY_GRP_TYPE_REQ,
+						pdu, sizeof(pdu),
+						read_by_grp_type_cb,
+						bt_gatt_request_ref(op),
+						async_req_unref);
 	} else {
 		uint8_t pdu[6 + get_uuid_len(uuid)];
 
 		if (uuid->type == BT_UUID_UNSPEC) {
 			free(op);
-			return false;
+			return NULL;
 		}
 
 		/* Discover by UUID */
@@ -833,23 +844,26 @@ static bool discover_services(struct bt_att *att, bt_uuid_t *uuid,
 		put_le16(start, pdu);
 		put_le16(end, pdu + 2);
 		put_le16(op->service_type, pdu + 4);
-		put_uuid_le(&op->uuid, pdu + 6);
+		bt_uuid_to_le(&op->uuid, pdu + 6);
 
-		result = bt_att_send(att, BT_ATT_OP_FIND_BY_TYPE_VAL_REQ,
-							pdu, sizeof(pdu),
-							find_by_type_val_cb,
-							discovery_op_ref(op),
-							discovery_op_unref);
+		op->id = bt_att_send(att, BT_ATT_OP_FIND_BY_TYPE_VAL_REQ,
+						pdu, sizeof(pdu),
+						find_by_type_val_cb,
+						bt_gatt_request_ref(op),
+						async_req_unref);
 	}
 
-	if (!result)
+	if (!op->id) {
 		free(op);
+		return NULL;
+	}
 
-	return result;
+	return bt_gatt_request_ref(op);
 }
 
-bool bt_gatt_discover_all_primary_services(struct bt_att *att, bt_uuid_t *uuid,
-					bt_gatt_discovery_callback_t callback,
+struct bt_gatt_request *bt_gatt_discover_all_primary_services(
+					struct bt_att *att, bt_uuid_t *uuid,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
@@ -858,9 +872,10 @@ bool bt_gatt_discover_all_primary_services(struct bt_att *att, bt_uuid_t *uuid,
 							destroy);
 }
 
-bool bt_gatt_discover_primary_services(struct bt_att *att, bt_uuid_t *uuid,
+struct bt_gatt_request *bt_gatt_discover_primary_services(
+					struct bt_att *att, bt_uuid_t *uuid,
 					uint16_t start, uint16_t end,
-					bt_gatt_discovery_callback_t callback,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
@@ -868,9 +883,10 @@ bool bt_gatt_discover_primary_services(struct bt_att *att, bt_uuid_t *uuid,
 								destroy, true);
 }
 
-bool bt_gatt_discover_secondary_services(struct bt_att *att, bt_uuid_t *uuid,
+struct bt_gatt_request *bt_gatt_discover_secondary_services(
+					struct bt_att *att, bt_uuid_t *uuid,
 					uint16_t start, uint16_t end,
-					bt_gatt_discovery_callback_t callback,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
@@ -879,7 +895,7 @@ bool bt_gatt_discover_secondary_services(struct bt_att *att, bt_uuid_t *uuid,
 }
 
 struct read_incl_data {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	struct bt_gatt_result *result;
 	int pos;
 	int ref_count;
@@ -893,7 +909,7 @@ static struct read_incl_data *new_read_included(struct bt_gatt_result *res)
 	if (!data)
 		return NULL;
 
-	data->op = discovery_op_ref(res->op);
+	data->op = bt_gatt_request_ref(res->op);
 	data->result = res;
 
 	return data;
@@ -913,7 +929,7 @@ static void read_included_unref(void *data)
 	if (__sync_sub_and_fetch(&read_data->ref_count, 1))
 		return;
 
-	discovery_op_unref(read_data->op);
+	async_req_unref(read_data->op);
 
 	free(read_data);
 }
@@ -925,8 +941,7 @@ static void read_included_cb(uint8_t opcode, const void *pdu,
 					uint16_t length, void *user_data)
 {
 	struct read_incl_data *data = user_data;
-	struct bt_gatt_result *final_result = NULL;
-	struct discovery_op *op = data->op;
+	struct bt_gatt_request *op = data->op;
 	uint8_t att_ecode = 0;
 	uint8_t read_pdu[2];
 	bool success;
@@ -963,7 +978,6 @@ static void read_included_cb(uint8_t opcode, const void *pdu,
 		last_handle = get_le16(data->result->pdu + data->pos -
 							data->result->data_len);
 		if (last_handle == op->end_handle) {
-			final_result = op->result_head;
 			success = true;
 			goto done;
 		}
@@ -972,12 +986,14 @@ static void read_included_cb(uint8_t opcode, const void *pdu,
 		put_le16(op->end_handle, pdu + 2);
 		put_le16(GATT_INCLUDE_UUID, pdu + 4);
 
-		if (bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
-				pdu, sizeof(pdu), discover_included_cb,
-				discovery_op_ref(op), discovery_op_unref))
+		op->id = bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
+						pdu, sizeof(pdu),
+						discover_included_cb,
+						bt_gatt_request_ref(op),
+						async_req_unref);
+		if (op->id)
 			return;
 
-		discovery_op_unref(op);
 		success = false;
 		goto done;
 	}
@@ -995,13 +1011,12 @@ static void read_included_cb(uint8_t opcode, const void *pdu,
 	success = false;
 
 done:
-	if (op->callback)
-		op->callback(success, att_ecode, final_result, op->user_data);
+	discovery_op_complete(op, success, att_ecode);
 }
 
 static void read_included(struct read_incl_data *data)
 {
-	struct discovery_op *op = data->op;
+	struct bt_gatt_request *op = data->op;
 	uint8_t pdu[2];
 
 	memcpy(pdu, data->result->pdu + 2, sizeof(uint16_t));
@@ -1014,17 +1029,16 @@ static void read_included(struct read_incl_data *data)
 							read_included_unref))
 		return;
 
-	read_included_unref(data);
-
 	if (op->callback)
 		op->callback(false, 0, NULL, data->op->user_data);
+
+	read_included_unref(data);
 }
 
 static void discover_included_cb(uint8_t opcode, const void *pdu,
 					uint16_t length, void *user_data)
 {
-	struct bt_gatt_result *final_result = NULL;
-	struct discovery_op *op = user_data;
+	struct bt_gatt_request *op = user_data;
 	struct bt_gatt_result *cur_result;
 	uint8_t att_ecode = 0;
 	uint16_t last_handle;
@@ -1092,40 +1106,38 @@ static void discover_included_cb(uint8_t opcode, const void *pdu,
 		put_le16(op->end_handle, pdu + 2);
 		put_le16(GATT_INCLUDE_UUID, pdu + 4);
 
-		if (bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
-							pdu, sizeof(pdu),
-							discover_included_cb,
-							discovery_op_ref(op),
-							discovery_op_unref))
+		op->id = bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
+						pdu, sizeof(pdu),
+						discover_included_cb,
+						bt_gatt_request_ref(op),
+						async_req_unref);
+		if (op->id)
 			return;
 
-		discovery_op_unref(op);
 		success = false;
 		goto failed;
 	}
 
 done:
 	success = true;
-	final_result = op->result_head;
 
 failed:
-	if (op->callback)
-		op->callback(success, att_ecode, final_result, op->user_data);
+	discovery_op_complete(op, success, att_ecode);
 }
 
-bool bt_gatt_discover_included_services(struct bt_att *att,
+struct bt_gatt_request *bt_gatt_discover_included_services(struct bt_att *att,
 					uint16_t start, uint16_t end,
-					bt_gatt_discovery_callback_t callback,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	uint8_t pdu[6];
 
 	if (!att)
 		return false;
 
-	op = new0(struct discovery_op, 1);
+	op = new0(struct bt_gatt_request, 1);
 	if (!op)
 		return false;
 
@@ -1139,25 +1151,23 @@ bool bt_gatt_discover_included_services(struct bt_att *att,
 	put_le16(end, pdu + 2);
 	put_le16(GATT_INCLUDE_UUID, pdu + 4);
 
-	if (!bt_att_send(att, BT_ATT_OP_READ_BY_TYPE_REQ,
-					pdu, sizeof(pdu),
-					discover_included_cb,
-					discovery_op_ref(op),
-					discovery_op_unref)) {
+	op->id = bt_att_send(att, BT_ATT_OP_READ_BY_TYPE_REQ, pdu, sizeof(pdu),
+				discover_included_cb, bt_gatt_request_ref(op),
+				async_req_unref);
+	if (!op->id) {
 		free(op);
-		return false;
+		return NULL;
 	}
 
-	return true;
+	return bt_gatt_request_ref(op);
 }
 
 static void discover_chrcs_cb(uint8_t opcode, const void *pdu,
 					uint16_t length, void *user_data)
 {
-	struct discovery_op *op = user_data;
+	struct bt_gatt_request *op = user_data;
 	bool success;
 	uint8_t att_ecode = 0;
-	struct bt_gatt_result *final_result = NULL;
 	size_t data_length;
 	uint16_t last_handle;
 
@@ -1187,8 +1197,8 @@ static void discover_chrcs_cb(uint8_t opcode, const void *pdu,
 
 	data_length = ((uint8_t *) pdu)[0];
 
-	if (((length - 1) % data_length) ||
-			(data_length != 7 && data_length != 21)) {
+	if ((data_length != 7 && data_length != 21) ||
+					((length - 1) % data_length)) {
 		success = false;
 		goto done;
 	}
@@ -1206,41 +1216,38 @@ static void discover_chrcs_cb(uint8_t opcode, const void *pdu,
 		put_le16(op->end_handle, pdu + 2);
 		put_le16(GATT_CHARAC_UUID, pdu + 4);
 
-		if (bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
+		op->id = bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
 						pdu, sizeof(pdu),
 						discover_chrcs_cb,
-						discovery_op_ref(op),
-						discovery_op_unref))
+						bt_gatt_request_ref(op),
+						async_req_unref);
+		if (op->id)
 			return;
 
-		discovery_op_unref(op);
 		success = false;
 		goto done;
 	}
 
 success:
-	final_result = op->result_head;
 	success = true;
 
 done:
-	if (op->callback)
-		op->callback(success, att_ecode, final_result,
-							op->user_data);
+	discovery_op_complete(op, success, att_ecode);
 }
 
-bool bt_gatt_discover_characteristics(struct bt_att *att,
+struct bt_gatt_request *bt_gatt_discover_characteristics(struct bt_att *att,
 					uint16_t start, uint16_t end,
-					bt_gatt_discovery_callback_t callback,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	uint8_t pdu[6];
 
 	if (!att)
 		return false;
 
-	op = new0(struct discovery_op, 1);
+	op = new0(struct bt_gatt_request, 1);
 	if (!op)
 		return false;
 
@@ -1254,22 +1261,21 @@ bool bt_gatt_discover_characteristics(struct bt_att *att,
 	put_le16(end, pdu + 2);
 	put_le16(GATT_CHARAC_UUID, pdu + 4);
 
-	if (!bt_att_send(att, BT_ATT_OP_READ_BY_TYPE_REQ,
-					pdu, sizeof(pdu),
-					discover_chrcs_cb,
-					discovery_op_ref(op),
-					discovery_op_unref)) {
+	op->id = bt_att_send(att, BT_ATT_OP_READ_BY_TYPE_REQ, pdu, sizeof(pdu),
+				discover_chrcs_cb, bt_gatt_request_ref(op),
+				async_req_unref);
+	if (!op->id) {
 		free(op);
-		return false;
+		return NULL;
 	}
 
-	return true;
+	return bt_gatt_request_ref(op);
 }
 
 static void read_by_type_cb(uint8_t opcode, const void *pdu,
 					uint16_t length, void *user_data)
 {
-	struct discovery_op *op = user_data;
+	struct bt_gatt_request *op = user_data;
 	bool success;
 	uint8_t att_ecode = 0;
 	size_t data_length;
@@ -1312,16 +1318,16 @@ static void read_by_type_cb(uint8_t opcode, const void *pdu,
 
 		put_le16(last_handle + 1, pdu);
 		put_le16(op->end_handle, pdu + 2);
-		put_uuid_le(&op->uuid, pdu + 4);
+		bt_uuid_to_le(&op->uuid, pdu + 4);
 
-		if (bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
+		op->id = bt_att_send(op->att, BT_ATT_OP_READ_BY_TYPE_REQ,
 						pdu, sizeof(pdu),
 						read_by_type_cb,
-						discovery_op_ref(op),
-						discovery_op_unref))
+						bt_gatt_request_ref(op),
+						async_req_unref);
+		if (op->id)
 			return;
 
-		discovery_op_unref(op);
 		success = false;
 		goto done;
 	}
@@ -1329,24 +1335,22 @@ static void read_by_type_cb(uint8_t opcode, const void *pdu,
 	success = true;
 
 done:
-	if (op->callback)
-		op->callback(success, att_ecode, success ? op->result_head :
-							NULL, op->user_data);
+	discovery_op_complete(op, success, att_ecode);
 }
 
 bool bt_gatt_read_by_type(struct bt_att *att, uint16_t start, uint16_t end,
 					const bt_uuid_t *uuid,
-					bt_gatt_discovery_callback_t callback,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	uint8_t pdu[4 + get_uuid_len(uuid)];
 
 	if (!att || !uuid || uuid->type == BT_UUID_UNSPEC)
 		return false;
 
-	op = new0(struct discovery_op, 1);
+	op = new0(struct bt_gatt_request, 1);
 	if (!op)
 		return false;
 
@@ -1359,25 +1363,25 @@ bool bt_gatt_read_by_type(struct bt_att *att, uint16_t start, uint16_t end,
 
 	put_le16(start, pdu);
 	put_le16(end, pdu + 2);
-	put_uuid_le(uuid, pdu + 4);
+	bt_uuid_to_le(uuid, pdu + 4);
 
-	if (!bt_att_send(att, BT_ATT_OP_READ_BY_TYPE_REQ, pdu, sizeof(pdu),
-					read_by_type_cb, discovery_op_ref(op),
-					discovery_op_unref)) {
-		free(op);
-		return false;
-	}
+	op->id = bt_att_send(att, BT_ATT_OP_READ_BY_TYPE_REQ, pdu, sizeof(pdu),
+						read_by_type_cb,
+						bt_gatt_request_ref(op),
+						async_req_unref);
+	if (op->id)
+		return true;
 
-	return true;
+	free(op);
+	return false;
 }
 
 static void discover_descs_cb(uint8_t opcode, const void *pdu,
 					uint16_t length, void *user_data)
 {
-	struct discovery_op *op = user_data;
+	struct bt_gatt_request *op = user_data;
 	bool success;
 	uint8_t att_ecode = 0;
-	struct bt_gatt_result *final_result = NULL;
 	uint8_t format;
 	uint16_t last_handle;
 	size_t data_length;
@@ -1432,40 +1436,38 @@ static void discover_descs_cb(uint8_t opcode, const void *pdu,
 		put_le16(last_handle + 1, pdu);
 		put_le16(op->end_handle, pdu + 2);
 
-		if (bt_att_send(op->att, BT_ATT_OP_FIND_INFO_REQ,
+		op->id = bt_att_send(op->att, BT_ATT_OP_FIND_INFO_REQ,
 						pdu, sizeof(pdu),
 						discover_descs_cb,
-						discovery_op_ref(op),
-						discovery_op_unref))
+						bt_gatt_request_ref(op),
+						async_req_unref);
+		if (op->id)
 			return;
 
-		discovery_op_unref(op);
 		success = false;
 		goto done;
 	}
 
 success:
-	final_result = op->result_head;
 	success = true;
 
 done:
-	if (op->callback)
-		op->callback(success, att_ecode, final_result, op->user_data);
+	discovery_op_complete(op, success, att_ecode);
 }
 
-bool bt_gatt_discover_descriptors(struct bt_att *att,
+struct bt_gatt_request *bt_gatt_discover_descriptors(struct bt_att *att,
 					uint16_t start, uint16_t end,
-					bt_gatt_discovery_callback_t callback,
+					bt_gatt_request_callback_t callback,
 					void *user_data,
 					bt_gatt_destroy_func_t destroy)
 {
-	struct discovery_op *op;
+	struct bt_gatt_request *op;
 	uint8_t pdu[4];
 
 	if (!att)
 		return false;
 
-	op = new0(struct discovery_op, 1);
+	op = new0(struct bt_gatt_request, 1);
 	if (!op)
 		return false;
 
@@ -1478,13 +1480,14 @@ bool bt_gatt_discover_descriptors(struct bt_att *att,
 	put_le16(start, pdu);
 	put_le16(end, pdu + 2);
 
-	if (!bt_att_send(att, BT_ATT_OP_FIND_INFO_REQ, pdu, sizeof(pdu),
+	op->id = bt_att_send(att, BT_ATT_OP_FIND_INFO_REQ, pdu, sizeof(pdu),
 						discover_descs_cb,
-						discovery_op_ref(op),
-						discovery_op_unref)) {
+						bt_gatt_request_ref(op),
+						async_req_unref);
+	if (!op->id) {
 		free(op);
-		return false;
+		return NULL;
 	}
 
-	return true;
+	return bt_gatt_request_ref(op);
 }
