@@ -36,7 +36,7 @@
 #include <readline/readline.h>
 #include <readline/history.h>
 
-#include <gobex/gobex.h>
+#include "gobex/gobex.h"
 #include "btio/btio.h"
 
 static GMainLoop *main_loop = NULL;
